package com.mixel.docusphere.service;

import com.mixel.docusphere.dto.request.UserRequestDTO;
import com.mixel.docusphere.dto.response.UserRespondDTO;
import com.mixel.docusphere.entity.User;
import com.mixel.docusphere.mapper.UserMapper;
import com.mixel.docusphere.repository.UserRepository;
import org.springframework.security.crypto.argon2.Argon2PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;

@Service
public class UserService {

    private final UserRepository userRepository;

    private final PasswordService passwordService;

    // Constructor
    public UserService(UserRepository userRepository, PasswordService passwordService) {
        this.userRepository = userRepository;
        this.passwordService = passwordService;
    }

    public List<UserRespondDTO> findAll() {
        return userRepository.findAll().stream()
                .map(UserMapper::toRespondDTO)
                .collect(Collectors.toList());
    }

    public Optional<UserRespondDTO> findById(UUID id) {
        return userRepository.findById(id)
                .map(UserMapper::toRespondDTO);
    }

    public UserRespondDTO save(UserRequestDTO userRequestDTO) {
        User user = UserMapper.toEntity(userRequestDTO);
        isPasswordAlreadySet(userRequestDTO, user);
        return UserMapper.toRespondDTO(userRepository.save(user));
    }

    public Optional<UserRespondDTO> update(UUID id, UserRequestDTO userRequestDTO){
        Optional<User> userOptional = userRepository.findById(id);
        if (userOptional.isPresent()) {
            User user = userOptional.get();
            user.setUsername(userRequestDTO.getUsername());
            user.setName(userRequestDTO.getName());
            user.setEmail(userRequestDTO.getEmail());

            isPasswordAlreadySet(userRequestDTO, user);
            return Optional.of(UserMapper.toRespondDTO(userRepository.save(user)));
        }
        return Optional.empty();
    }

    private void isPasswordAlreadySet(UserRequestDTO userRequestDTO, User user) {
        if (userRequestDTO.getPassword() != null && !userRequestDTO.getPassword().isEmpty()) {
            user.setPasswordHash(passwordService.hashPassword(userRequestDTO.getPassword()));
        }
    }

    public void deleteById(UUID id) {
        userRepository.deleteById(id);
    }
}